program define anovalator
*! version 2.0 -- 04/19/10 -- add 2way alias for twoway
*! version 1.9 -- 04/15/10 -- add predict() option
*! version 1.8 -- 03/10/10 -- tweak twoway, 3way
*! version 1.7.2- 02/19/10 -- add 3way option
*! version 1.6 -- 02/18/10 -- fix twoway error
*! version 1.5 -- 02/17/10 -- improved pairwise output
*! version 1.4 -- 02/13/10 -- add simple (main effects) option
*! version 1.3 -- 02/12/10 -- simplify maineffects
*! version 1.2 -- 02/05/10 -- add twoway & pairwise options
*! version 1.0 -- 01/27/10 -- Phil Ender

/* "Behold, I am created anovalator, destroyer of models"          */
/* caution: this program is considered experimental                */
/* there is a minimal help file, there are no returned results ... */
/*      ... there are few internal consistency checks, be afraid   */
/* see http://www.ats.ucla.edu/stat/stata/faq/anovalator.htm       */ 
/* note: at option allows only one value for each variable         */
/* note: option q for quiet margins                                */

  syntax varlist [ , Fratio wgt(string) MAINeffect TWOway 2way 3way ///
    simple Quietly PAIRwise at(passthru) PRedict(passthru) ]
  version 11.0
  if "`wgt'"=="" & "`maineffect'"=="" & "`twoway'"=="" & "`pairwise'"=="" & "`simple'"=="" & "`3way'"=="" & "`2way'"=="" {
    display as txt "No options selected.  Choose one of the following: wgt(), MAINeffect, TWOway, 3way, simple or PAIRwise."
  }
  tempname me_m1
  estimates store `me_m1'
  local wcount=wordcount("`varlist'")
  if "`maineffect'"=="maineffect" {        /* maineffect option */
    quietly margins `varlist', asbalanced `at' `predict' post
    foreach term of varlist `varlist' {
      quietly testparm i.`term', equal 
      display
      display as txt "anovalator main-effect for " as res "`term' `at' `predict'"
      display as txt "chi2(" as res r(df) as txt ") = " as res r(chi2) ///
        as txt "   p-value = " as res chi2tail(r(df), r(chi2))
      if "`fratio'"=="fratio" {
        display as txt "scaled as F-ratio = " as res r(chi2)/r(df)
      }
    }
    quietly estimates restore `me_m1'
  }                                        /* end maineffect */
  if "`wgt'"!="" {                         /* wgt option */
    if `wcount'>1 {
      display as err "Too many variables selected for wgt() option"
      exit
    }
    `quietly' margins `varlist', asbalanced post `predict' `at'
    quietly levelsof(`varlist'), local(mar) 
    local len=wordcount("`mar'")
    forvalues i = 1/`len' {
      local m`i'=word("`mar'",`i')
    }
    local len2=wordcount("`wgt'")
    forvalues i= 1/`len2' {
      local w`i' = word("`wgt'", `i')
      if strpos("`w`i''", "-")==0 & strpos("`w`i''", "+")==0 {
        local w`i' = "+" + "`w`i''"
      } 
    }
    if `len'==`len2' {
      local linspec ""
      forvalues i=1/`len' {
        local linspec = "`linspec'" +  "`w`i''*_b[`m`i''.`varlist']"
      }
      display
      display as txt "anovalator contrast for " as res "`varlist' `at' `predict'"
      display
      lincom `linspec'
    }
    else {
      display as txt "Number of weights in wgt() does not equal number of levels of variable " as res "`varlist'."
    }
    quietly estimates restore `me_m1'
  }                                        /* end wgt */
  if "`pairwise'"=="pairwise" {            /* pairwise option */
    if `wcount'>1 {
      display as err "Too many variables selected for pairwise option"
      exit
    }
    `quietly' margins `varlist', asbalanced `at' `predict' post
    quietly levelsof `varlist', local(mar)
    local len=wordcount("`mar'")
    local lenm1 = `len' - 1
    forvalues i = 1/`len' {
      local m`i'=word("`mar'",`i')
    }
    display
    display as txt "anovalator pairwise comparisons for " as res "`varlist' `at' `predict'"
    display
    display as txt "Comparison" _col(21)  "Coef.   Std. Err.      z    P>|z|     [95% Conf. Interval]"
    forvalues i = 1/`lenm1' {
      local k = `i'+1
      forvalues j = `k'/`len' {
        quietly lincom `m`i''.`varlist' - `m`j''.`varlist'
        local zscore =r(estimate)/r(se)
        local crit=1.96
        local hi = r(estimate)+`crit'*r(se)
        local lo = r(estimate)-`crit'*r(se)
        local pvalue=(1-normal(abs(`zscore')))*2
        display as txt "`m`i'' vs `m`j''" _col(15) as res %11.6g r(estimate) %11.6g r(se) ///
           " " %8.3g `zscore' "   " %05.3f `pvalue' " " %12.7g `lo'  %12.7g `hi'
      }
    }                                          
    quietly estimates restore `me_m1'
  }                                              /* end pairwise */
  if "`twoway'"=="twoway" | "`2way'"=="2way" {   /* twoway option */
    if `wcount'~=2 {
      display as err "twoway option requires two variables"
      exit
    }
    tokenize `varlist'
    quietly levelsof `1', local(mval)
    local first=word("`mval'",1)
    local mval=subinstr("`mval'","`first' ","",1)
    quietly margins `1', dydx(`2') asbalanced `at' `predict' post
    local xvars=e(xvars)
    foreach eq of local xvars {
      if strpos("`eq'","b.")==0 {
        foreach term of local mval {
          quietly test [`eq']`first'.`1'-[`eq']`term'.`1'=0, accum
        }
      }
    }
    display
    display as txt "anovalator two-way interaction for " as res "`1'#`2' `at' `predict'"
    display as txt "chi2(" as res r(df) as txt ") = " as res r(chi2) ///
            as txt "   p-value = " as res chi2tail(r(df), r(chi2))
    if "`fratio'"=="fratio" {
      display as txt "scaled as F-ratio = " as res r(chi2)/r(df)
    }
    quietly estimates restore `me_m1'
  }                                            /* end twoway */
  if "`simple'"=="simple" {                    /* simple (main effects) option */
    if `wcount'~=2 {
        display as err "simple option requires two variables"
        exit
      }
    tokenize `varlist'
    quietly levelsof `2', local(lof)
    display
    foreach svalue of local lof {
      if "`at'"=="" {
        local at2 "at(`2'=`svalue')"
      }
      else {
        local at2 = subinstr("`at'","at(","at(`2'=`svalue' ",1)
      }
      quietly margins, dydx(`1') `at2' asbalanced `predict' post
      display as txt "anovalator test of simple main effects for " as res "`1' `at2' `predict'"
      quietly testparm i.`1'
      display as txt "chi2(" as res r(df) as txt ") = " as res r(chi2) ///
              as txt "   p-value = " as res chi2tail(r(df), r(chi2))
      if "`fratio'"=="fratio" {
        display as txt "scaled as F-ratio = " as res r(chi2)/r(df)
      }
      display
      quietly estimates restore `me_m1'
    }
  }                                          /* end simple (main effects) */
  if "`3way'"=="3way" {                      /* 3way option  */
    if `wcount'~=3 {
      display as err "3way option requires 3 variables"
      exit
    }
    tokenize `varlist'
    quietly margins `1'#`2', dydx(`3') asbal `at' `predict' post
    quietly levelsof `1', local(t1)
    quietly levelsof `2', local(t2)
    local a1=word("`t1'",1)
    local b1=word("`t2'",1)
    local t1=subinstr("`t1'","`a1' ","",1)
    local t2=subinstr("`t2'","`b1' ","",1)
    local xvars=e(xvars)
    foreach eq of local xvars {
      if strpos("`eq'","b.")==0 {
       foreach aterm of local t1 {
        foreach bterm of local t2 {
          quietly test [`eq']`a1'.`1'#`b1'.`2'-[`eq']`a1'.`1'#`bterm'.`2'-[`eq']`aterm'.`1'#`b1'.`2'+[`eq']`aterm'.`1'#`bterm'.`2'=0, accum
        }
       }
      }
    } 
    display
    display as txt "anovalator 3-way interaction for " as res "`1'#`2'#`3' `at' `predict'"
    display as txt "chi2(" as res r(df) as txt ") = " as res r(chi2) ///
            as txt "   p-value = " as res chi2tail(r(df), r(chi2))
    if "`fratio'"=="fratio" {
      display as txt "scaled as F-ratio = " as res r(chi2)/r(df)
    }
    quietly estimates restore `me_m1'
  }                                          /* end 3way     */
end
